from torchvision import transforms
from ocr_tools.util import CTCLabelConverter, load_dict_from_file, replace_punctuation2
import os
import torch
import torch.nn.functional as F
from PIL import Image
import copy
import cv2
import onnxruntime
import numpy as np
from ocr_tools.unet_masker import UnetMasker
class OCRProcessor():
    def __init__(self, deep_model_path, label_file, mask_model_path = None):
        #注释deepocr pth模型
        # self.model = torch.load(os.path.join(model_path, "dldeepOCR.pth"))
        #self.toTensor = transforms.ToTensor()
        # 该用deepocr onnx模型
        self.toTensor = transforms.ToTensor()
        self.onnx_sess = onnxruntime.InferenceSession(deep_model_path)
        self.onnx_input_name = self.onnx_sess.get_inputs()[0].name
        self.onnx_output_name = self.onnx_sess.get_outputs()[0].name
        self.character = load_dict_from_file(label_file)
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.converter = CTCLabelConverter(self.character)
        self.imgH = 32
        if mask_model_path is not None:
            self.unet_masker = UnetMasker(mask_model_path, img_height=64)
            self.space_threshold = 0.25


    def preprocess(self, img):
        images = [img]
        resized_images = []
        for image in images:
            w, h = image.size
            scale = h * 1.0 / self.imgH
            new_width = int(w / scale)
            if new_width == 0:
                return None
            resized_image = image.resize((new_width, self.imgH), Image.BICUBIC)
            if new_width < 32:
                back_img = np.zeros((self.imgH, 32))
                img = np.asarray(resized_image)
                back_img[0:self.imgH, 0:new_width] = img
                resized_image = Image.fromarray(back_img)

            img = self.toTensor(resized_image)
            img.sub_(0.5).div_(0.5)
            Pad_img = img
            resized_images.append(Pad_img)
        image_tensors = torch.cat([t.unsqueeze(0) for t in resized_images], 0)
        return image_tensors

    def predict_cvimg(self, cv_img):
        cv_img = cv2.cvtColor(cv_img, cv2.COLOR_BGR2RGB)
        img = Image.fromarray(cv_img)
        img = img.convert("L")
        return self.predict(img)


    def predict(self, img:Image):
        img = img.convert("L")

        ##注释deepocr pth调用方式
        # image_tensors = self.preprocess(img)
        # text_for_pred = torch.LongTensor(1, 80 + 1).fill_(0).to(self.device)
        # preds = self.model(image_tensors, text_for_pred)[0]

        # 使用deepocr onnx调用方式
        image_tensors = self.preprocess(img)
        if image_tensors is None:
            return "", 0
        image_tensors =image_tensors.numpy()
        preds = self.onnx_sess.run([self.onnx_output_name], {self.onnx_input_name: image_tensors})[0]
        preds = torch.from_numpy(preds)

        _, preds_index = preds.max(2)
        preds_size = torch.IntTensor([preds.size(1)] * 1)
        preds_prob = F.softmax(preds, dim=2)
        preds_max_prob, _ = preds_prob.max(dim=2)
        preds_str, detail_list = self.converter.decode(preds_index, preds_max_prob, preds_size)
        return preds_str[0][0], preds_str[0][1], detail_list

    def post_mask_process(self, cv_mask, img:Image):

        w, h = img.size
        scale = h * 1.0 / self.imgH
        new_width = int(w / scale)
        if new_width == 0:
            return None
        resized_image = img.resize((new_width, self.imgH), Image.BICUBIC)
        width, height = resized_image.size
        mask = cv2.resize(cv_mask.astype(np.float32), (width, height))
        bg_mask = np.zeros((height + 32, width))
        bg_mask[16:16 + height, :] = mask
        bg_mask = bg_mask.astype(np.uint8)

        kerner = np.ones((16, 3), np.uint8)
        res_mask = cv2.dilate(bg_mask, kerner)
        res_mask = cv2.erode(res_mask, kerner)
        res_mask = np.where(res_mask > 127, 1, 0)
        return res_mask, mask

    def get_space_info(self, one_zero_mask):
        height, width = one_zero_mask.shape
        width_np = np.sum(one_zero_mask, axis=0)
        width_list = width_np.tolist()
        width_list = sorted(width_list)
        sub_list = width_list[int(0.9 * len(width_list)):]
        mean_height = int(sum(sub_list) / len(sub_list))
        bx = -1
        space_flag=False
        res = []
        for idx, ele in enumerate(width_np):
            if ele == 0:
                if bx == -1:
                    bx = idx
                    space_flag = True
                else:
                    if not space_flag:
                        space_flag = True
                        bx = idx
            else:
                if space_flag:
                    if bx > 0:
                        dwidth = idx - bx
                        if dwidth > mean_height * self.space_threshold:
                            res.append((bx*1.0 / width, idx*1.0 / width))
                space_flag = False
        return res

    def space_process(self, word_details, space_info):
        word_detail = []
        for ele in word_details:
            if len(ele[0].strip()) > 0:
                word_detail.append(ele)
        for ele in space_info:
            word_detail = self.merge_space(word_detail, ele)

        return word_detail
    def merge_space(self, word_detail:list, space_ele):
        if len(word_detail) < 2:
            return word_detail
        left, right = space_ele
        space_bean = (" ", 1, space_ele)
        for idx, word_bean in enumerate(word_detail[0:-1]):
            if word_bean[2][0] > right:
                break
            next_word_bean = word_detail[idx+1]
            #空格正好位于两个字符之间
            if word_bean[2][1] <= left and next_word_bean[2][0] >= right:
                word_detail.insert(idx+1, space_bean)
                return word_detail
            #和两个字符都有重合
            if word_bean[2][1] > left and next_word_bean[2][1] < right:
                word_detail.insert(idx + 1, space_bean)
                return word_detail
            #只和左侧重合
            if word_bean[2][1] > left and word_bean[2][0] < right:
                if left < word_bean[2][0]:
                    #左侧插入
                    word_detail.insert(idx, space_bean)
                else:
                    #右侧插入
                    word_detail.insert(idx+1, space_bean)
                return word_detail
            #只和右侧重合
            if next_word_bean[2][1] > left and next_word_bean[2][0] < right:
                if left < next_word_bean[2][0]:
                    #左侧插入
                    word_detail.insert(idx+1, space_bean)
                else:
                    #右侧插入
                    word_detail.insert(idx+2, space_bean)
                return word_detail

        return word_detail

    def predict_space(self, img: Image):
        cv_mask = self.unet_masker.get_mask(copy.copy(img))

        one_zero_mask, mask = self.post_mask_process(cv_mask, img)

        space_info = self.get_space_info(one_zero_mask)

        text, score, detail_list = self.predict(img)

        detail_list = self.space_process(word_details=detail_list, space_info=space_info)
        text = ""
        sum_score = 0
        for ele in detail_list:
            text+=ele[0]
            sum_score+=ele[1]

        return text, sum_score / len(detail_list), detail_list


def main():
    mask_model_path = "/home/xiaohong/PythonProject2022/Pytorch-UNet-master/checkpoints/unet-p64-mod.onnx"
    # deep_model_path = '/home/xiaohong/Pictures/A4/models/deepocr/deep-ocr-model-v2.onnx'
    # deep_label_path = "/home/xiaohong/Pictures/A4/models/deepocr/char_7442_ocr.txt"
    deep_model_path = '/home/xiaohong/Documents/deepocr/dldeep-ocr-model-7456-v5.onnx'
    deep_label_path = "/home/xiaohong/Documents/deepocr/new_dict_7456.txt"
    ocr_handler = OCRProcessor(deep_model_path, deep_label_path, mask_model_path=mask_model_path)
    import time
    input_folder = "/home/xiaohong/Pictures/ocr评估数据集/img/英文空格截图数据"
    save_path = "/home/xiaohong/Pictures/ocr评估数据集/res"

    files = os.listdir(input_folder)

    fw = open(os.path.join(save_path, "labels.txt"), "w+")
    cost = 0
    cnt = 0
    label_map = load_label(os.path.join(input_folder, "labels.txt"))
    corr_cnt = 0
    space_corr_cnt = 0
    for file in files:
        # file = '1354.png'
        if file.__contains__("labels.txt"):
            continue
        img_path = os.path.join(input_folder, file)
        label = label_map[file]
        # img_path = "/mnt/data/xiaohong/声像项目/ocr-error/ysy_img_20.jpg"
        img = Image.open(img_path)
        # img = img.convert("L")
        # img = cv2.imread(img_path)
        print(file)
        start = time.time()*1000
        # pred_str, score, detail = ocr_handler.predict_cvimg(img)

        pred_str, score, detail = ocr_handler.predict_space(img)
        file_name = file
        # re_str = replace_punctuation2(pred_str, "")
        # if len(re_str) > 0 and re_str.encode("utf-8").isalnum():
            # shutil.copyfile(img_path, os.path.join(save_path, file_name))
        fw.write(file_name+" "+pred_str.strip()+"\n")
        # label_res = replace_punctuation2(label, "").replace(" ", "").strip()
        # pred_res = replace_punctuation2(pred_str, "").replace(" ", "").strip()
        pred_str = replace_punctuation2(pred_str, "")
        label_str = replace_punctuation2(label, "")
        label_res = label_str.strip()
        pred_res = pred_str.strip()
        if label_res == pred_res:
            space_corr_cnt += 1

        if label_res.replace(" ", "") == pred_res.replace(" ",""):
            corr_cnt+=1
        else:
            print(f"file:{file} label:{label_res} pred_str:{pred_res}  score:{score}")
        # pred_str, score = detector.en_ocr(img)
        cost += (time.time() * 1000 - start)

        cnt += 1
    end = time.time()
    fw.close()
    print(f"平均耗时：{(cost) * 1.0 / cnt}毫秒！ OCR准确率：{corr_cnt / cnt}, 包含空格准确率：{space_corr_cnt / cnt}")


def load_label(label_path):
    lines = [ele.strip() for ele in open(label_path, "r+").readlines()]
    label_map = {}
    for line in lines:
        linearr = line.split()
        file_name = linearr[0]
        file_label = line.replace(file_name, "").strip()
        label_map[file_name] = file_label
    return label_map



if __name__ == "__main__":
    # im = Image.open("/home/xiaohong/Pictures/ocr评估数据集/1.jpg")
    # deep_handle = OCRProcessor(model_path="/home/xiaohong/Pictures/A4/models/deepocr")
    # simpred, score = deep_handle.predict(im)
    # print("dddd")

    main()
