# -*- coding: utf-8 -*-
# @Time    : 2019/8/23 21:59
# @Author  : zhoujun
import json
import pathlib
import time
import os
import glob
import matplotlib.pyplot as plt
import numpy as np
import torch
import random
import re
import math

device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')


def load_dict_from_file(path):
    lines = open(path, "r+").readlines()
    res = ""
    for line in lines:
        res += line.strip()
    return res + " "


class CTCLabelConverter(object):
    """ Convert between text-label and text-index """

    def __init__(self, character):
        # character (str): set of the possible characters.
        dict_character = list(character)

        self.dict = {}
        for i, char in enumerate(dict_character):
            # NOTE: 0 is reserved for 'CTCblank' token required by CTCLoss
            self.dict[char] = i + 1

        self.character = ['[CTCblank]'] + dict_character  # dummy '[CTCblank]' token for CTCLoss (index 0)

    def encode(self, text, batch_max_length=25):
        """convert text-label into text-index.
        input:
            text: text labels of each image. [batch_size]
            batch_max_length: max length of text label in the batch. 25 by default

        output:
            text: text index for CTCLoss. [batch_size, batch_max_length]
            length: length of each text. [batch_size]
        """
        length = [len(s) for s in text]

        # The index used for padding (=0) would not affect the CTC loss calculation.
        batch_text = torch.LongTensor(len(text), batch_max_length).fill_(0)
        for i, t in enumerate(text):
            text = list(t)
            text = [self.dict[char] for char in text]
            batch_text[i][:len(text)] = torch.LongTensor(text)
        return (batch_text.to(device), torch.IntTensor(length).to(device))

    def decode(self, text_index, preds_max_prob, length):
        """ convert text-index into text-label. """
        texts = []
        detail_list = []
        for index, l in enumerate(length):
            t = text_index[index, :]

            char_list = []
            score_list = []
            size = l.item()
            begin_end_pos = 0.5 / (size - 1)
            for i in range(l):
                if t[i] != 0 and (not (i > 0 and t[i - 1] == t[i])):  # removing repeated characters and blank.
                    wchar = self.character[t[i]]
                    char_list.append(wchar)
                    wscore = preds_max_prob[index, i].item()
                    score_list.append(wscore)
                    pos = None
                    if i == 0:
                        pos = (0.001 / size, begin_end_pos)
                    elif i == size - 1:
                        pos = (1 - begin_end_pos, 1)
                    else:
                        pos = (begin_end_pos + (i - 1) / (size - 1), begin_end_pos + i / (size - 1))
                    detail_list.append((wchar, wscore, pos))
            text = ''.join(char_list)
            score = 0 if len(score_list) == 0 else sum(score_list) / len(score_list)
            texts.append((text, score))
        return texts, detail_list

    def cat_text(self, text, text_back):
        text_numpy = text.data
        text_back_numpy = text_back.data
        res = []
        for idx in range(len(text_numpy)):

            line = text_numpy[idx]
            line_back = text_back_numpy[idx]
            label = np.zeros((len(line) * 2,))
            label_idx = 0
            merge = torch.cat((line, line_back))
            # merge = np.hstack((line, line_back))
            for ele in merge:
                if ele > 0:
                    label[label_idx] = ele
                    label_idx += 1
            res.append(label)
        return torch.tensor(res, dtype=torch.int64)

    def reverse_text(self, text):
        text_numpy = text.numpy()
        res = []
        for idx in range(len(text_numpy)):
            line = text_numpy[idx]
            label = np.zeros((len(line),))
            label_idx = 0
            for ele in line:
                if ele > 0.5:
                    label[label_idx] = ele
                    label_idx += 1
            res.append(label)
        return torch.tensor(res, dtype=torch.int64)


def get_file_list(folder_path: str, p_postfix: list = None, sub_dir: bool = True) -> list:
    """
    获取所给文件目录里的指定后缀的文件,读取文件列表目前使用的是 os.walk 和 os.listdir ，这两个目前比 pathlib 快很多
    :param filder_path: 文件夹名称
    :param p_postfix: 文件后缀,如果为 [.*]将返回全部文件
    :param sub_dir: 是否搜索子文件夹
    :return: 获取到的指定类型的文件列表
    """
    assert os.path.exists(folder_path) and os.path.isdir(folder_path)
    if p_postfix is None:
        p_postfix = ['.jpg']
    if isinstance(p_postfix, str):
        p_postfix = [p_postfix]
    file_list = [x for x in glob.glob(folder_path + '/**/*.*', recursive=True) if
                 os.path.splitext(x)[-1] in p_postfix or '.*' in p_postfix]
    return file_list


def setup_logger(log_file_path: str = None):
    import logging
    logging._warn_preinit_stderr = 0
    logger = logging.getLogger('DBNet.pytorch')
    formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s: %(message)s')
    ch = logging.StreamHandler()
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    if log_file_path is not None:
        file_handle = logging.FileHandler(log_file_path)
        file_handle.setFormatter(formatter)
        logger.addHandler(file_handle)
    logger.setLevel(logging.DEBUG)
    return logger


# --exeTime
def exe_time(func):
    def newFunc(*args, **args2):
        t0 = time.time()
        back = func(*args, **args2)
        print("{} cost {:.3f}s".format(func.__name__, time.time() - t0))
        return back

    return newFunc


def load(file_path: str):
    file_path = pathlib.Path(file_path)
    func_dict = {'.txt': _load_txt, '.json': _load_json, '.list': _load_txt}
    assert file_path.suffix in func_dict
    return func_dict[file_path.suffix](file_path)


def _load_txt(file_path: str):
    with open(file_path, 'r', encoding='utf8') as f:
        content = [x.strip().strip('\ufeff').strip('\xef\xbb\xbf') for x in f.readlines()]
    return content


def _load_json(file_path: str):
    with open(file_path, 'r', encoding='utf8') as f:
        content = json.load(f)
    return content


def save(data, file_path):
    file_path = pathlib.Path(file_path)
    func_dict = {'.txt': _save_txt, '.json': _save_json}
    assert file_path.suffix in func_dict
    return func_dict[file_path.suffix](data, file_path)


def _save_txt(data, file_path):
    """
    将一个list的数组写入txt文件里
    :param data:
    :param file_path:
    :return:
    """
    if not isinstance(data, list):
        data = [data]
    with open(file_path, mode='w', encoding='utf8') as f:
        f.write('\n'.join(data))


def _save_json(data, file_path):
    with open(file_path, 'w', encoding='utf-8') as json_file:
        json.dump(data, json_file, ensure_ascii=False, indent=4)


def show_img(imgs: np.ndarray, title='img'):
    color = (len(imgs.shape) == 3 and imgs.shape[-1] == 3)
    imgs = np.expand_dims(imgs, axis=0)
    for i, img in enumerate(imgs):
        plt.figure()
        plt.title('{}_{}'.format(title, i))
        plt.imshow(img, cmap=None if color else 'gray')
    plt.show()


def draw_bbox_path(img_path, result, color=(255, 0, 0), thickness=2):
    if isinstance(img_path, str):
        img_path = cv2.imread(img_path)
        # img_path = cv2.cvtColor(img_path, cv2.COLOR_BGR2RGB)
    img_path = img_path.copy()
    for point in result:
        point = point.astype(int)
        cv2.polylines(img_path, [point], True, color, thickness)
    return img_path


def draw_bbox(img_path, result, color=(255, 0, 0), thickness=2):
    if isinstance(img_path, str):
        img_path = cv2.imread(img_path)
        # img_path = cv2.cvtColor(img_path, cv2.COLOR_BGR2RGB)
    img_path = img_path.copy()
    result = np.array(result)
    for point in result:
        point = point.astype(int)
        cv2.polylines(img_path, [point], True, color, thickness)
    return img_path


def cal_text_score(texts, gt_texts, training_masks, running_metric_text, thred=0.5):
    training_masks = training_masks.data.cpu().numpy()
    pred_text = texts.data.cpu().numpy() * training_masks
    pred_text[pred_text <= thred] = 0
    pred_text[pred_text > thred] = 1
    pred_text = pred_text.astype(np.int32)
    gt_text = gt_texts.data.cpu().numpy() * training_masks
    gt_text = gt_text.astype(np.int32)
    running_metric_text.update(gt_text, pred_text)
    score_text, _ = running_metric_text.get_scores()
    return score_text


def order_points_clockwise(pts):
    rect = np.zeros((4, 2), dtype="float32")
    s = pts.sum(axis=1)
    rect[0] = pts[np.argmin(s)]
    rect[2] = pts[np.argmax(s)]
    diff = np.diff(pts, axis=1)
    rect[1] = pts[np.argmin(diff)]
    rect[3] = pts[np.argmax(diff)]
    return rect


def order_points_clockwise_list(pts):
    pts = pts.tolist()
    pts.sort(key=lambda x: (x[1], x[0]))
    pts[:2] = sorted(pts[:2], key=lambda x: x[0])
    pts[2:] = sorted(pts[2:], key=lambda x: -x[0])
    pts = np.array(pts)
    return pts


def get_datalist(train_data_path):
    """
    获取训练和验证的数据list
    :param train_data_path: 训练的dataset文件列表，每个文件内以如下格式存储 ‘path/to/img\tlabel’
    :return:
    """
    train_data = []
    for p in train_data_path:
        with open(p, 'r', encoding='utf-8') as f:
            for line in f.readlines():
                line = line.strip('\n').replace('.jpg ', '.jpg\t').split('\t')
                if len(line) > 1:
                    img_path = pathlib.Path(line[0].strip(' '))
                    label_path = pathlib.Path(line[1].strip(' '))
                    if img_path.exists() and img_path.stat().st_size > 0 and label_path.exists() and label_path.stat().st_size > 0:
                        train_data.append((str(img_path), str(label_path)))
    return train_data


def save_result(result_path, box_list, score_list, is_output_polygon):
    if is_output_polygon:
        with open(result_path, 'wt') as res:
            for i, box in enumerate(box_list):
                box = box.reshape(-1).tolist()
                result = ",".join([str(int(x)) for x in box])
                score = score_list[i]
                res.write(result + ',' + str(score) + "\n")
    else:
        with open(result_path, 'wt') as res:
            for i, box in enumerate(box_list):
                score = score_list[i]
                box = box.reshape(-1).tolist()
                result = ",".join([str(int(x)) for x in box])
                # res.write(result + ',' + str(score) + "\n")
                res.write(result + "\n")


def expand_polygon(polygon):
    """
    对只有一个字符的框进行扩充
    """
    (x, y), (w, h), angle = cv2.minAreaRect(np.float32(polygon))
    if angle < -45:
        w, h = h, w
        angle += 90
    new_w = w + h
    box = ((x, y), (new_w, h), angle)
    points = cv2.boxPoints(box)
    return order_points_clockwise(points)


def nms(fp_boxes, cf_score, threshold=0.8):
    # If no bounding boxes, return empty list
    if len(fp_boxes) == 0:
        return [], []

    # Bounding boxes
    bounding_boxes = []
    bounding_fp_boxes = []
    confidence_score = []
    for pps_score in zip(fp_boxes, cf_score):
        pps = pps_score[0]
        score = pps_score[1]
        minx = min(pps[0][0], pps[1][0], pps[2][0], pps[3][0])
        miny = min(pps[0][1], pps[1][1], pps[2][1], pps[3][1])
        maxx = max(pps[0][0], pps[1][0], pps[2][0], pps[3][0])
        maxy = max(pps[0][1], pps[1][1], pps[2][1], pps[3][1])
        if maxy > miny and maxx > minx:
            bounding_boxes.append([minx, miny, maxx, maxy])
            confidence_score.append(score)
            bounding_fp_boxes.append(pps)
    boxes = np.array(bounding_boxes)

    # coordinates of bounding boxes
    start_x = boxes[:, 0]
    start_y = boxes[:, 1]
    end_x = boxes[:, 2]
    end_y = boxes[:, 3]

    # Confidence scores of bounding boxes
    score = np.array(confidence_score)

    # Picked bounding boxes
    picked_boxes = []
    picked_fp_boxes = []
    picked_score = []

    # Compute areas of bounding boxes
    areas = (end_x - start_x + 1) * (end_y - start_y + 1)

    # Sort by confidence score of bounding boxes
    order = np.argsort(score)

    # Iterate bounding boxes
    while order.size > 0:
        # The index of largest confidence score
        index = order[-1]

        # Pick the bounding box with largest confidence score
        picked_boxes.append(bounding_boxes[index])
        picked_fp_boxes.append(bounding_fp_boxes[index])
        picked_score.append(confidence_score[index])

        # Compute ordinates of intersection-over-union(IOU)
        x1 = np.maximum(start_x[index], start_x[order[:-1]])
        x2 = np.minimum(end_x[index], end_x[order[:-1]])
        y1 = np.maximum(start_y[index], start_y[order[:-1]])
        y2 = np.minimum(end_y[index], end_y[order[:-1]])

        # Compute areas of intersection-over-union
        w = np.maximum(0.0, x2 - x1 + 1)
        h = np.maximum(0.0, y2 - y1 + 1)
        intersection = w * h

        # Compute the ratio between intersection and union
        ratio = intersection / (areas[index] + areas[order[:-1]] - intersection)

        left = np.where(ratio < threshold)
        order = order[left]

    return picked_fp_boxes, picked_score


def area(box):
    box_b = np.array(box).astype(np.float32)
    rect = cv2.minAreaRect(box_b)
    rect_h_w = rect[1]
    return rect_h_w[0] * rect_h_w[1]


def iou(src_rect, dst_rect):
    dst = [
        [dst_rect[0], dst_rect[1]],
        [dst_rect[0] + dst_rect[2], dst_rect[1]],
        [dst_rect[0] + dst_rect[2], dst_rect[1] + dst_rect[3]], [dst_rect[0], dst_rect[1] + dst_rect[3]]]

    item = [[src_rect[0], src_rect[1]], [src_rect[0] + src_rect[2], src_rect[1]],
            [src_rect[0] + src_rect[2], src_rect[1] + src_rect[3]], [src_rect[0], src_rect[1] + src_rect[3]]]
    original_grasp_bboxes = np.array([np.array(dst).astype(np.int32)])
    max1 = max(original_grasp_bboxes.reshape(-1))
    prediction_grasp_bboxes = np.array([np.array(item).astype(np.int32)])
    max2 = max(prediction_grasp_bboxes.reshape(-1))
    whsize = max(max1, max2) + 1
    im = np.zeros((whsize, whsize), dtype="uint8")
    im1 = np.zeros((whsize, whsize), dtype="uint8")
    original_grasp_mask = cv2.fillPoly(im, original_grasp_bboxes, 255)
    prediction_grasp_mask = cv2.fillPoly(im1, prediction_grasp_bboxes, 255)
    masked_and = cv2.bitwise_and(original_grasp_mask, prediction_grasp_mask, mask=im)
    masked_or = cv2.bitwise_or(original_grasp_mask, prediction_grasp_mask)
    or_area = np.sum(np.float32(np.greater(masked_or, 0)))
    and_area = np.sum(np.float32(np.greater(masked_and, 0)))
    IOU = and_area / or_area
    return IOU


def iou_rotate(dst, item):
    original_grasp_bboxes = np.array([np.array(dst).astype(np.int32)])
    max1 = max(original_grasp_bboxes.reshape(-1))
    prediction_grasp_bboxes = np.array([np.array(item).astype(np.int32)])
    max2 = max(prediction_grasp_bboxes.reshape(-1))
    whsize = max(max1, max2) + 1
    im = np.zeros((whsize, whsize), dtype="uint8")
    im1 = np.zeros((whsize, whsize), dtype="uint8")
    original_grasp_mask = cv2.fillPoly(im, original_grasp_bboxes, 255)
    prediction_grasp_mask = cv2.fillPoly(im1, prediction_grasp_bboxes, 255)
    masked_and = cv2.bitwise_and(original_grasp_mask, prediction_grasp_mask, mask=im)
    masked_or = cv2.bitwise_or(original_grasp_mask, prediction_grasp_mask)
    or_area = np.sum(np.float32(np.greater(masked_or, 0)))
    and_area = np.sum(np.float32(np.greater(masked_and, 0)))
    IOU = and_area / or_area
    return IOU


def crop(img, box):
    box_b = np.array(box).astype(np.float32)
    rotated_rect = cv2.minAreaRect(box_b)
    width = int(rotated_rect[1][0])
    height = int(rotated_rect[1][1])
    src_pts = cv2.boxPoints(rotated_rect)
    if width > height:
        w = width
        h = height
        dst_pts = np.array([
            [0, h - 1],
            [0, 0],
            [w - 1, 0], [w - 1, h - 1]], dtype="float32")
    else:
        w = height
        h = width
        dst_pts = np.array([
            [0, 0], [w - 1, 0], [w - 1, h - 1], [0, h - 1]], dtype="float32")

    M = cv2.getPerspectiveTransform(src_pts, dst_pts)
    warped = cv2.warpPerspective(img, M, (w, h))
    return warped


def crop_rect(img, box):
    new_img = img[box[1]:box[1] + box[3], box[0]:box[0] + box[2]]
    return new_img


def expand_box(box):
    box_b = np.array(box).astype(np.float32)
    rect = cv2.minAreaRect(box_b)
    rect_w_h = rect[1]
    expand_min = min(rect[1])
    ratio = min(1, math.log10(max(rect[1]) / expand_min))
    newh = rect_w_h[1]
    neww = rect_w_h[0]
    if neww >= newh:
        newh = newh + newh * 0.44 * ratio
        neww = neww + newh * 0.84 * ratio
    else:
        newh = newh + neww * 0.84 * ratio
        neww = neww + neww * 0.44 * ratio

    new_rect = (rect[0], (neww, newh), rect[2])
    # new_rect[1] = (newh, neww)
    points = cv2.boxPoints(new_rect)
    p1 = [int(points[0][0]), int(points[0][1])]
    p2 = [int(points[1][0]) + 1, int(points[1][1])]
    p3 = [int(points[2][0]) + 1, int(points[2][1]) + 1]
    p4 = [int(points[3][0]), int(points[3][1]) + 1]
    return [p1, p2, p3, p4]


def ceilg_32(num):
    if num % 32 == 0:
        return num
    return (int(num / 32) + 1) * 32


def draw_rect(img, rect):
    color1 = random.randint(10, 250)
    color2 = random.randint(10, 250)
    color3 = random.randint(10, 250)
    img2 = cv2.rectangle(img, (rect[0], rect[1]), (rect[2], rect[3]), (color1, color2, color3), 2)
    return img2


def draw_rects(img, rects):
    color1 = random.randint(10, 250)
    color2 = random.randint(10, 250)
    color3 = random.randint(10, 250)
    for rect in rects:
        img2 = cv2.rectangle(img, (rect[0], rect[1]), (rect[2] + rect[0], rect[1] + rect[3]), (color1, color2, color3),
                             2)
    return img2


def draw_bboxes(img, rects):
    color1 = random.randint(10, 250)
    color2 = random.randint(10, 250)
    color3 = random.randint(10, 250)
    for rect in rects:
        img2 = cv2.rectangle(img, (rect[0], rect[1]), (rect[2], rect[3]), (color1, color2, color3), 2)
    return img2


from PIL import ImageDraw, Image, ImageFont


def draw_infos(cv_img, infos):
    new_img = Image.fromarray(cv2.cvtColor(cv_img, cv2.COLOR_BGR2RGB))
    draw = ImageDraw.Draw(new_img)
    font = "/home/xiaohong/PythonProjectOCR/StepdeepOcr1/font/chinese/16.ttf"

    for item in infos:
        color = (0, 0, 255)
        text_color = (255, 0, 0)
        image_font = ImageFont.truetype(font=font, size=int(int(item.box[3]) * 0.76))
        draw.line([item.rotate_box[0][0], item.rotate_box[0][1], item.rotate_box[1][0], item.rotate_box[1][1]],
                  fill=color)
        draw.line([item.rotate_box[1][0], item.rotate_box[1][1], item.rotate_box[2][0], item.rotate_box[2][1]],
                  fill=color)
        draw.line([item.rotate_box[2][0], item.rotate_box[2][1], item.rotate_box[3][0], item.rotate_box[3][1]],
                  fill=color)
        draw.line([item.rotate_box[0][0], item.rotate_box[0][1], item.rotate_box[3][0], item.rotate_box[3][1]],
                  fill=color)
        # draw.rectangle((item.box[0], item.box[1], item.box[0] + item.box[2], item.box[1]+item.box[3]), fill=color)
        draw.text((item.box[0], item.box[1]), item.text,
                  font=image_font,
                  fill=text_color)
    cv_img = cv2.cvtColor(np.asarray(new_img), cv2.COLOR_RGB2BGR)
    return cv_img


def draw_rotate_rects(img, points_list):
    color1 = random.randint(10, 250)
    color2 = random.randint(10, 250)
    color3 = random.randint(10, 250)
    img2 = img.copy()
    for points in points_list:
        img2 = cv2.line(img2, (points[0][0], points[0][1]), (points[1][0], points[1][1]), (color1, color2, color3), 2)
        img2 = cv2.line(img2, (points[1][0], points[1][1]), (points[2][0], points[2][1]), (color1, color2, color3), 2)
        img2 = cv2.line(img2, (points[2][0], points[2][1]), (points[3][0], points[3][1]), (color1, color2, color3), 2)
        img2 = cv2.line(img2, (points[0][0], points[0][1]), (points[3][0], points[3][1]), (color1, color2, color3), 2)
    return img2


def draw_rotate_rect(img, points):
    color1 = random.randint(10, 250)
    color2 = random.randint(10, 250)
    color3 = random.randint(10, 250)
    img2 = cv2.line(img, (points[0][0], points[0][1]), (points[1][0], points[1][1]), (color1, color2, color3), 2)
    img2 = cv2.line(img2, (points[1][0], points[1][1]), (points[2][0], points[2][1]), (color1, color2, color3), 2)
    img2 = cv2.line(img2, (points[2][0], points[2][1]), (points[3][0], points[3][1]), (color1, color2, color3), 2)
    img2 = cv2.line(img2, (points[0][0], points[0][1]), (points[3][0], points[3][1]), (color1, color2, color3), 2)
    return img2


def replace_punctuation(line):
    punctuation = """！？｡＂＃＄％＆＇（）＊＋－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃》「」『』【】〔〕
    〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘'‛“”„‟…,.!?':"“，、。！@#￥%……&Z*‧（）{}[]【】﹏"""
    re_punctuation = "[{}]+".format(punctuation)
    line = re.sub(re_punctuation, "", line)
    return line


def get_iou_boxes(items, shape):
    if len(items) <= 1:
        return items
    res = []
    dst = items[0]
    res.append(dst)
    for item in items[1:]:
        if iou(dst, item, shape) > 0.6:
            res.append(item)
    return res


def sum_area(items):
    area_cnt = 0
    for item in items:
        area_cnt += area(item['box'])
    return area_cnt


def check_audio_text(text, reg_ocr, persons):
    check_flag = False
    reg_res = re.search(reg_ocr, text)
    if reg_res and len(reg_res.regs) > 0:
        sub_match = reg_res.regs[1:]
        new_text = text
        cnt = 0
        for item in sub_match:
            sub_text = text[item[0]:item[1]]
            new_text = new_text.replace(sub_text, persons[cnt], 1)
            cnt += 1
        check_flag = True
        return new_text, check_flag
    else:
        return text, check_flag


def horizontal_iou(rect1, rect2):
    if rect1[0] > rect2[0] + rect2[2] or rect1[0] + rect1[2] < rect2[0]:
        return 0
    width = min(rect1[2], rect2[2])
    maxb = max(rect2[0], rect1[0])
    mine = min(rect1[0] + rect1[2], rect2[0] + rect2[2])
    horizontal = mine - maxb + 1
    if horizontal > 0:
        return horizontal * 1.0 / width
    return 0


def horizontal_dis(rect1, rect2):
    maxb = max(rect2[0], rect1[0])
    mine = min(rect1[0] + rect1[2], rect2[0] + rect2[2])
    horizontal = mine - maxb + 1
    if horizontal > 0:
        return 0
    return math.fabs(horizontal)


def vertical_dis(rect1, rect2):
    maxb = max(rect2[1], rect1[1])
    mine = min(rect1[1] + rect1[3], rect2[1] + rect2[3])
    vertical = mine - maxb + 1
    if vertical > 0:
        return 0
    return math.fabs(vertical)


def comp_box(item1, item2):
    box1 = item1.box
    box2 = item2.box
    if vertical_iou(box1, box2) > 0.6:
        return box1[0] - box2[0]
    return box1[1] - box2[1]


def merge_rect(boxes):
    if len(boxes) > 0:
        bx = boxes[0][0]
        by = boxes[0][1]
        ex = boxes[0][0] + boxes[0][2]
        ey = boxes[0][1] + boxes[0][3]
    for box in boxes[1:]:
        if box[0] < bx:
            bx = box[0]
        if box[1] < by:
            by = box[1]
        if box[0] + box[2] > ex:
            ex = box[0] + box[2]
        if box[1] + box[3] > ey:
            ey = box[1] + box[3]
    return [bx, by, ex - bx, ey - by]


def dis_rect(rect_ele, rect_max):
    max_bx = max(rect_ele[0], rect_max[0])
    min_ex = min(rect_ele[0] + rect_ele[2], rect_max[0] + rect_max[2])
    return max_bx - min_ex


def merge_pair_list(box_list):
    points = []
    for ele in box_list:
        points.extend(ele[0])
    rotate_box = cv2.minAreaRect(np.array(points).astype(np.float32))
    src_pts = cv2.boxPoints(rotate_box).astype(np.int32)
    box_rect = cv2.boundingRect(src_pts)
    return (src_pts, box_rect, box_rect[2] * box_rect[3])


def merge_box_pair_list(box_pair_list):
    res_box_list = []
    while (len(box_pair_list) <= 1):
        res_box_list.extend(box_pair_list)
        return res_box_list
    while (len(box_pair_list) > 1):
        tmp_box_list = []
        max_pair = box_pair_list[0]
        tmp_box_list.append(max_pair)
        for ele in box_pair_list[1:]:
            rect_max = max_pair[1]
            rect_ele = ele[1]
            if vertical_iou(rect_max, rect_ele) > 0.7 and dis_rect(rect_ele, rect_max) < min(rect_ele[3], rect_max[3]):
                tmp_box_list.append(ele)
        res_box_list.append(merge_pair_list(tmp_box_list))
        for ele2 in tmp_box_list:
            # print(ele2)
            box_pair_list.remove(ele2)
        res_box_list.extend(merge_box_pair_list(box_pair_list))
    return res_box_list


def mere_rotate_box_by_horizontal(boxes):
    box_pair_list = []
    for ele_box in boxes:
        dd_box = ele_box.tolist()
        ele_box = np.array(ele_box, dtype=np.float32)
        # new_box = expand_box(ele_box)
        bounding_box = cv2.boundingRect(ele_box)
        box_pair_list.append((dd_box, bounding_box, bounding_box[2] * bounding_box[3]))
    box_pair_list = sorted(box_pair_list, key=lambda x: x[2], reverse=True)

    new_box_list = merge_box_pair_list(box_pair_list)
    new_boxes = []
    for boxele in new_box_list:
        new_boxes.append(boxele[0])

    return np.array(new_boxes)


def convert_boundingbox_rotatebox(rect_box):
    bx, by, width, height = rect_box
    return [[bx, by], [bx + width, by], [bx + width, by + height], [bx, by + height]]


def vertical_iou(rect1, rect2):
    height = min(rect1[3], rect2[3])
    maxb = max(rect2[1], rect1[1])
    mine = min(rect1[1] + rect1[3], rect2[1] + rect2[3])
    vertical = mine - maxb + 1
    if vertical > 0:
        return vertical * 1.0 / height
    return 0


def find_lcsubstr(s1, s2):
    m = [[0 for i in range(len(s2) + 1)] for j in range(len(s1) + 1)]  # 生成0矩阵，为方便后续计算，比字符串长度多了一列
    mmax = 0  # 最长匹配的长度
    p = 0  # 最长匹配对应在s1中的最后一位
    for i in range(len(s1)):
        for j in range(len(s2)):
            if s1[i] == s2[j]:
                m[i + 1][j + 1] = m[i][j] + 1
                if m[i + 1][j + 1] > mmax:
                    mmax = m[i + 1][j + 1]
                    p = i + 1
    return s1[p - mmax:p], mmax


def get_the_same_lenth_str_by_min_edit_dis(str1, str2, padstr):
    lenstr1 = len(str1)
    lenstr2 = len(str2)
    if lenstr1 == 0 or lenstr2 == 0:
        if lenstr1 == 0 and lenstr2 == 0:
            return "", ""
        if lenstr1 == 0:
            res1 = ""
            for i in range(lenstr2):
                res1 += padstr
            return res1, str2
        res2 = ""
        for i in range(lenstr1):
            res2 += padstr
        return str1, res2
    else:
        share_sub_seq, seq_len = find_lcsubstr(str1, str2)
        if seq_len == 0:
            max_len = max(lenstr1, lenstr2)
            for i in range(max_len - lenstr1):
                str1 += padstr
            for i in range(max_len - lenstr2):
                str2 += padstr
            return str1, str2
        else:
            str1_pre = str1[0: str1.find(share_sub_seq)]
            str2_pre = str2[0: str2.find(share_sub_seq)]
            if len(str1_pre) > 0 or len(str2_pre) > 0:
                str1_pre, str2_pre = get_the_same_lenth_str_by_min_edit_dis(str1_pre, str2_pre, padstr)
            str1_tail = str1[str1.find(share_sub_seq) + seq_len:lenstr1]
            str2_tail = str2[str2.find(share_sub_seq) + seq_len:lenstr2]
            if len(str1_tail) > 0 or len(str2_tail) > 0:
                str1_tail, str2_tail = get_the_same_lenth_str_by_min_edit_dis(str1_tail, str2_tail, padstr)

            return str1_pre + share_sub_seq + str1_tail, str2_pre + share_sub_seq + str2_tail


def is_chinese(str1):
    res = True
    for w in str1:
        if not '\u4e00' <= w <= '\u9fff':
            res = False
            break
    return res


def replace_punctuation2(line, padstr):
    punctuation = "！？｡＂＃＄％＆＇（）＊/_-＋－／：；＜＝＞＠［＼］＾＿;｀｛｜｝’～｟｠｢｣､''、〃》「」『』【】〔〕 〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘'‛“”,„‟…,.!?':)\"“，、。！@#￥%……&Z*‧(（）{}[]【】.﹏".replace(
        " ", "")
    for pp in punctuation:
        line = line.replace(pp, padstr)
    return line

def resize_image(img, short_size):
    height, width, _ = img.shape
    if height < width:
        new_height = short_size
        new_width = new_height / height * width
    else:
        new_width = short_size
        new_height = new_width / width * height
    new_height = int(round(new_height / 32) * 32)
    new_width = int(round(new_width / 32) * 32)
    resized_img = cv2.resize(img, (new_width, new_height))
    return resized_img


def resize_image_by_short(img, short_size):
    height, width, _ = img.shape
    if min(width, height) < short_size:
        return img
    if height < width:
        new_height = short_size
        new_width = new_height / height * width
    else:
        new_width = short_size
        new_height = new_width / width * height
    new_height = int(round(new_height / 32) * 32)
    new_width = int(round(new_width / 32) * 32)
    resized_img = cv2.resize(img, (new_width, new_height))
    return resized_img


def resize_image_with_height(img, new_height):
    height, width, _ = img.shape
    scale = new_height * 1.0 / height
    new_width = int(width * scale)
    resized_img = cv2.resize(img, (new_width, new_height))
    return resized_img


def sorted_box_pair(p1, p2):
    rect1 = p1[1]
    rect2 = p2[1]
    if vertical_iou(rect1, rect2) > 0.5:
        return rect1[0] - rect2[0]
    else:
        return rect1[1] - rect2[1]


def merge_sub_img(sub_img_list, split_cv_img):
    new_height_img_list = []
    total_width = 0
    split_width = split_cv_img.shape[1]
    for img in sub_img_list:
        tmp_img = resize_image_with_height(img, 32)
        shape = tmp_img.shape
        if len(shape) == 2:
            tmp_img = cv2.cvtColor(tmp_img, cv2.COLOR_GRAY2BGR)
        total_width += shape[1]
        new_height_img_list.append(tmp_img)
        new_height_img_list.append(split_cv_img)
        total_width += split_width
    new_img = np.zeros((32, total_width, 3))
    bx = 0

    for ele in new_height_img_list:
        shape = ele.shape
        new_img[0:32, bx:bx + shape[1], 0:shape[2]] = ele
        bx += shape[1]
    return new_img


def trim_space_by_dot(text):
    dots = "`~!@#$%^&*()-+=_[]{;:'\"},./<>?\\"
    for w in dots:
        while (text.__contains__(" " + w)):
            text = text.replace(" " + w, w)
        while (text.__contains__(w + " ")):
            text = text.replace(w + " ", w)
    return text


def verdis_horizen_iou(bbox_pair, tmp_list):
    for ele in tmp_list:
        box1 = bbox_pair[0]
        box2 = ele[0]
        if horizontal_iou(box1, box2) > 0.8 and vertical_dis(box1, box2) < min(box1[3], box2[3]):
            return True
    return False


def merge_caption_by_relative_position(res_pair):
    """
    [(box, img, text),....]
    :param res_pair:
    :return:
    """
    text_list = []
    while (len(res_pair) > 1):
        tmp_list = [res_pair[0]]
        for bbox_pair in res_pair[1:]:
            if verdis_horizen_iou(bbox_pair, tmp_list):
                tmp_list.append(bbox_pair)
        label = ""
        for ele in tmp_list:
            res_pair.remove(ele)
            label = label + " " + ele[1]
        text_list.append(label)
        if len(tmp_list) > 1:
            print("合并字幕：" + label)
        tmp_list.clear()

    if len(res_pair) > 0:
        text_list.append(res_pair[0][1])
    return text_list


def merge_boxes_by_relative_position(bbox_list):
    """
    [(box, img, text),....]
    :param res_pair:
    :return:
    """
    boxes_res_list = []
    while (len(bbox_list) > 1):
        tmp_list = [bbox_list[0]]
        for bbox in bbox_list[1:]:
            if vertical_iou(bbox, tmp_list[0]) > 0.6 and horizontal_dis(bbox, tmp_list[0]) < 2 * max(bbox[3],
                                                                                                     tmp_list[0][3]):
                tmp_list.append(bbox)
        label = ""
        for ele in tmp_list:
            bbox_list.remove(ele)
        boxes_res_list.append(merge_rect(tmp_list))
        if len(tmp_list) > 1:
            print("合并字幕：" + label)
        tmp_list.clear()

    if len(bbox_list) > 0:
        boxes_res_list.append(merge_rect(bbox_list))
    return boxes_res_list


if __name__ == '__main__':

    import cv2

    ddd = "xxxxx, xxx . , 44s,c% ,^ &&&&,dsd"
    ddd = trim_space_by_dot(ddd)

    ddd = [3, 4, 5]
    npddd = np.array(ddd)
    ddds = np.array(npddd)
    base_dir = "/mnt/data/xiaohong/亚速营相关/网络图"
    save_dir = "/mnt/data/xiaohong/亚速营相关/ysy-label/new_img"
    cnt = 0
    for file in os.listdir(base_dir):
        img_path = os.path.join(base_dir, file)
        img = cv2.imread(img_path)
        cv2.imwrite(save_dir + "/ysy_" + str(cnt) + ".jpg", img)
        cnt += 1

    print("---over---")

    # str1 = "ctcBackWordStr.substring"
    # str2 = "ctcWordStr.substring(ctcW"
    #
    # isalnum = str2.encode("utf-8").isalnum()
    # print(is_chinese("纯中文，！"))
    # new_str1, new_str2 = get_the_same_lenth_str_by_min_edit_dis(str1, str2, "#")

    # print(f"{new_str1} : {new_str2}")
    # img = np.zeros((1, 3, 640, 640))
    # show_img(img[0][0])
    # dst = [[10, 10], [20, 15], [20, 20], [10, 20]]
    # src = [[10, 10], [20, 10], [20, 20], [10, 20]]
    # dst = [10, 12, 20, 15]
    # src = [10, 10, 20, 10]
    # iou1 = iou(src, dst)
    # print(iou1)
    # ocr_res = "习近平签署第一0八号，一0九号主席令"
    # reg_ocr, persons = num2re(ocr_res)

    # import re
    #
    # sentence = "国务院总理李克d日前签署国w院令，公布修订后的中华人民共和国水下文物保护管理条例。"
    # # ress = sentence.replace("中国", "dd", 1)
    # # pattern = "你好(.{5})中国"
    # pattern = "(.{3})日前签署(.{3})令"
    # persons = ["李克强", "国务院"]
    # match_res = re.search(pattern, sentence, flags=0)
    # new_text, check_flag = check_audio_text(sentence, pattern, persons)
    #
    # print(new_text)
